@@ -40,6 +40,12 @@ const (
 	ESC
 	CtrlSpace
 
+	// https://apple.stackexchange.com/questions/24261/how-do-i-send-c-that-is-control-slash-to-the-terminal
+	CtrlBackSlash
+	CtrlRightBracket
+	CtrlCaret
+	CtrlSlash
+
 	Invalid
 	Resize
 	Mouse
@@ -117,7 +123,7 @@ func (c Color) is24() bool {
 
 const (
 	colUndefined Color = -2
-	colDefault         = -1
+	colDefault   Color = -1
 )
 
 const (
@@ -164,13 +170,11 @@ func (p ColorPair) Bg() Color {
 	return p.bg
 }
 
-func (p ColorPair) is24() bool {
-	return p.fg.is24() || p.bg.is24()
-}
-
 type ColorTheme struct {
 	Fg           Color
 	Bg           Color
+	PreviewFg    Color
+	PreviewBg    Color
 	DarkBg       Color
 	Gutter       Color
 	Prompt       Color
@@ -219,6 +223,8 @@ type BorderStyle struct {
 	bottomRight rune
 }
 
+type BorderCharacter int
+
 func MakeBorderStyle(shape BorderShape, unicode bool) BorderStyle {
 	if unicode {
 		return BorderStyle{
@@ -242,6 +248,17 @@ func MakeBorderStyle(shape BorderShape, unicode bool) BorderStyle {
 	}
 }
 
+func MakeTransparentBorder() BorderStyle {
+	return BorderStyle{
+		shape:       BorderAround,
+		horizontal:  ' ',
+		vertical:    ' ',
+		topLeft:     ' ',
+		topRight:    ' ',
+		bottomLeft:  ' ',
+		bottomRight: ' '}
+}
+
 type Renderer interface {
 	Init()
 	Pause(clear bool)
@@ -257,7 +274,7 @@ type Renderer interface {
 	MaxY() int
 	DoesAutoWrap() bool
 
-	NewWindow(top int, left int, width int, height int, borderStyle BorderStyle) Window
+	NewWindow(top int, left int, width int, height int, preview bool, borderStyle BorderStyle) Window
 }
 
 type Window interface {
@@ -319,12 +336,16 @@ var (
 	ColInfo            ColorPair
 	ColHeader          ColorPair
 	ColBorder          ColorPair
+	ColPreview         ColorPair
+	ColPreviewBorder   ColorPair
 )
 
 func EmptyTheme() *ColorTheme {
 	return &ColorTheme{
 		Fg:           colUndefined,
 		Bg:           colUndefined,
+		PreviewFg:    colUndefined,
+		PreviewBg:    colUndefined,
 		DarkBg:       colUndefined,
 		Gutter:       colUndefined,
 		Prompt:       colUndefined,
@@ -348,8 +369,10 @@ func init() {
 	Default16 = &ColorTheme{
 		Fg:           colDefault,
 		Bg:           colDefault,
+		PreviewFg:    colUndefined,
+		PreviewBg:    colUndefined,
 		DarkBg:       colBlack,
-		Gutter:       colBlack,
+		Gutter:       colUndefined,
 		Prompt:       colBlue,
 		Match:        colGreen,
 		Current:      colYellow,
@@ -363,6 +386,8 @@ func init() {
 	Dark256 = &ColorTheme{
 		Fg:           colDefault,
 		Bg:           colDefault,
+		PreviewFg:    colUndefined,
+		PreviewBg:    colUndefined,
 		DarkBg:       236,
 		Gutter:       colUndefined,
 		Prompt:       110,
@@ -378,6 +403,8 @@ func init() {
 	Light256 = &ColorTheme{
 		Fg:           colDefault,
 		Bg:           colDefault,
+		PreviewFg:    colUndefined,
+		PreviewBg:    colUndefined,
 		DarkBg:       251,
 		Gutter:       colUndefined,
 		Prompt:       25,
@@ -410,6 +437,8 @@ func initTheme(theme *ColorTheme, baseTheme *ColorTheme, forceBlack bool) {
 	}
 	theme.Fg = o(baseTheme.Fg, theme.Fg)
 	theme.Bg = o(baseTheme.Bg, theme.Bg)
+	theme.PreviewFg = o(theme.Fg, o(baseTheme.PreviewFg, theme.PreviewFg))
+	theme.PreviewBg = o(theme.Bg, o(baseTheme.PreviewBg, theme.PreviewBg))
 	theme.DarkBg = o(baseTheme.DarkBg, theme.DarkBg)
 	theme.Gutter = o(theme.DarkBg, o(baseTheme.Gutter, theme.Gutter))
 	theme.Prompt = o(baseTheme.Prompt, theme.Prompt)
@@ -446,6 +475,8 @@ func initPalette(theme *ColorTheme) {
 		ColInfo = pair(theme.Info, theme.Bg)
 		ColHeader = pair(theme.Header, theme.Bg)
 		ColBorder = pair(theme.Border, theme.Bg)
+		ColPreview = pair(theme.PreviewFg, theme.PreviewBg)
+		ColPreviewBorder = pair(theme.Border, theme.PreviewBg)
 	} else {
 		ColPrompt = pair(colDefault, colDefault)
 		ColNormal = pair(colDefault, colDefault)
@@ -460,6 +491,8 @@ func initPalette(theme *ColorTheme) {
 		ColInfo = pair(colDefault, colDefault)
 		ColHeader = pair(colDefault, colDefault)
 		ColBorder = pair(colDefault, colDefault)
+		ColPreview = pair(colDefault, colDefault)
+		ColPreviewBorder = pair(colDefault, colDefault)
 	}
 }
 
